#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

import json
from datetime import datetime, timedelta, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Ge, Le, Or
from pyramid.decorator import reify
from zope.interface import Interface, implementer
from zope.intid.interfaces import IIntIds
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.booking.interfaces import IBookingContainer
from pyams_app_msc.feature.planning import IPlanning
from pyams_app_msc.feature.planning.interfaces import ISession, VERSION_INFO, VERSION_INFO_ABBR
from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_app_msc.shared.catalog.interfaces import ICatalogEntry
from pyams_app_msc.shared.catalog.portlet.skin.interfaces import ICatalogViewItemsPortletCalendarRendererSettings, \
    ICatalogViewItemsPortletPanelsRendererSettings
from pyams_app_msc.shared.theater import IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.audience import get_audience_contact
from pyams_app_msc.shared.theater.interfaces import SESSION_REQUEST_MODE
from pyams_catalog.query import CatalogResultSet, IsNone
from pyams_content.feature.filter.container import FilterContainer
from pyams_content.feature.filter.interfaces import IAggregatedPortletRendererSettings
from pyams_content.shared.view.portlet.interfaces import IViewItemsPortletSettings
from pyams_content.shared.view.portlet.skin import ViewItemsPortletPanelsRenderer, \
    ViewItemsPortletPanelsRendererSettings
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import canonical_url, relative_url

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


#
# Catalog view items portlet calendar renderer
#

@factory_config(ICatalogViewItemsPortletCalendarRendererSettings)
@implementer(IAggregatedPortletRendererSettings)
class CatalogViewItemsPortletCalendarRendererSettings(FilterContainer):
    """Catalog view items portlet calendar renderer settings"""

    filters_css_class = FieldProperty(ICatalogViewItemsPortletCalendarRendererSettings['filters_css_class'])
    calendar_css_class = FieldProperty(ICatalogViewItemsPortletCalendarRendererSettings['calendar_css_class'])
    sessions_weeks = FieldProperty(ICatalogViewItemsPortletCalendarRendererSettings['sessions_weeks'])


@adapter_config(name='catalog-calendar',
                required=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/view-catalog-calendar.pt',
                 layer=IPyAMSLayer)
class CatalogViewItemsPortletCalendarRenderer(ViewItemsPortletPanelsRenderer):
    """Catalog view items portlet calendar renderer"""

    label = _("MSC: Catalog entries month calendar")
    weight = 110

    settings_interface = ICatalogViewItemsPortletCalendarRendererSettings
    use_authentication = True

    @reify
    def audience(self):
        """Audience getter"""
        return self.request.params.get('audience')

    def get_calendar_config(self, results):
        now = tztime(datetime.now(timezone.utc))
        end = now + timedelta(weeks=self.renderer_settings.sessions_weeks)
        theater = get_parent(self.context, IMovieTheater)
        translate = self.request.localizer.translate
        config = {
            'contentHeight': 'auto',
            'validRange': {
                'start': now.isoformat(),
                'end': end.isoformat()
            },
            'buttonText': {
                'today': translate(_("Today")),
                'month': translate(_("Month")),
                'week': translate(_("Week")),
                'day': translate(_("Day")),
                'list': translate(_("List")),
                'all-day': translate(_("All-day")),
                'prev': " « ",
                'next': " » "
            }
        }
        events = []
        audience = self.audience
        for result in results:
            planning = IPlanning(result, None)
            if planning is None:
                continue
            for session in planning.values():
                if (not session.extern_bookable) or (session.start_date < now) or (session.start_date > end):
                    continue
                if audience:
                    audiences = session.audiences
                    if (not audiences) and IWfCatalogEntry.providedBy(result):
                        audiences = result.audiences or ()
                    if audience not in audiences:
                        continue
                events.append({
                    'title': II18n(result).query_attribute('title', request=self.request),
                    'href': relative_url(result, self.request,
                                         display_context=theater,
                                         view_name='booking-new.html',
                                         query={
                                             'session_id': ICacheKeyValue(session),
                                             'audience': audience
                                         }),
                    'start': session.start_date.isoformat(),
                    'end': session.end_date.isoformat()
                })
        config['events'] = events
        return json.dumps(config)

    def get_session_request_url(self, context):
        theater = get_parent(self.context, IMovieTheater)
        settings = IMovieTheaterSettings(theater)
        if not settings.allow_session_request:
            return None
        if settings.session_request_mode == SESSION_REQUEST_MODE.FORM.value:
            return canonical_url(context, self.request, 'session-request.html', {
                'audience': self.audience
            })
        contact_email = get_audience_contact(theater, self.audience)
        return f'mailto:{contact_email}'


#
# Catalog view items portlet panels renderer
#

@factory_config(ICatalogViewItemsPortletPanelsRendererSettings)
@implementer(IAggregatedPortletRendererSettings)
class CatalogViewItemsPortletPanelsRendererSettings(FilterContainer, ViewItemsPortletPanelsRendererSettings):
    """Catalog view items portlet panels renderer settings"""

    filters_css_class = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['filters_css_class'])
    results_css_class = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['results_css_class'])
    first_panel_css_class = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['first_panel_css_class'])
    panels_css_class = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['panels_css_class'])
    display_sessions = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['display_sessions'])
    sessions_weeks = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['sessions_weeks'])
    display_free_seats = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['display_free_seats'])


@adapter_config(name='catalog-panels',
                required=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/view-catalog-panels.pt',
                 layer=IPyAMSLayer)
class CatalogViewItemsPortletPanelsRenderer(ViewItemsPortletPanelsRenderer):
    """Catalog view items portlet catalog panels renderer"""

    label = _("MSC: Catalog entries with next planned sessions")
    weight = 120

    settings_interface = ICatalogViewItemsPortletPanelsRendererSettings
    use_authentication = True

    @reify
    def audience(self):
        """Audience getter"""
        return self.request.params.get('audience')

    def get_sessions(self, wf_entry):
        """Display incoming sessions for provided catalog entry"""
        entry = get_parent(wf_entry, ICatalogEntry)
        catalog = get_utility(ICatalog)
        intids = get_utility(IIntIds)
        now = tztime(datetime.now(timezone.utc))
        params = [
            Eq(catalog['object_types'], get_interface_base_name(ISession)),
            Eq(catalog['parents'], intids.queryId(entry)),
            Ge(catalog['planning_start_date'], now),
            Le(catalog['planning_end_date'], now + timedelta(weeks=self.renderer_settings.sessions_weeks))
        ]
        if self.audience:
            params.append(Or(Eq(catalog['planning_audience'], self.audience),
                             IsNone(catalog['planning_audience'])))
        query = And(*params)
        yield from sorted(filter(lambda x: x.extern_bookable,
                                 CatalogResultSet(CatalogQuery(catalog).query(query))),
                          key=lambda x: x.start_date)

    def get_version(self, session):
        """Get displayed version of provided entry"""
        if not session.version:
            return ''
        translate = self.request.localizer.translate
        return translate(VERSION_INFO_ABBR.get(VERSION_INFO(session.version), _("(undefined)")))

    @staticmethod
    def get_free_seats(session):
        """Get free seats for provided session"""
        bookings = IBookingContainer(session, None)
        if bookings is None:
            return session.capacity, session.capacity
        return session.capacity - bookings.get_confirmed_seats(), session.capacity

    def get_session_request_url(self, context):
        theater = get_parent(context, IMovieTheater)
        settings = IMovieTheaterSettings(theater)
        if not settings.allow_session_request:
            return None
        if settings.session_request_mode == SESSION_REQUEST_MODE.FORM.value:
            return canonical_url(context, self.request, 'session-request.html', {
                'audience': self.audience
            })
        theater = get_parent(self.context, IMovieTheater)
        contact_email = get_audience_contact(theater, self.request.params.get('audience'))
        return f'mailto:{contact_email}'



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyramid_mailer.message import Attachment
from zope.dublincore.interfaces import IZopeDublinCore

from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.shared.catalog.interfaces import ICatalogEntry
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.mail import IMailTemplates
from pyams_file.interfaces.thumbnail import IThumbnails
from pyams_i18n.interfaces import II18n
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_mail.message import HTMLMessage
from pyams_security.interfaces import ISecurityManager
from pyams_utils.date import format_date, format_datetime
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_workflow.interfaces import IWorkflowVersions
from pyams_zmi.utils import get_object_label


def get_booking_message_values(context, request, view):
    """Booking message values getter"""
    theater = get_parent(context, IMovieTheater)
    # generate templates values
    session = ISession(context)
    entry = get_parent(session, ICatalogEntry)
    if entry is not None:
        versions = IWorkflowVersions(entry, None)
        if versions is not None:
            entry = versions.get_version(-1)
    values = {
        'theater_name': II18n(theater).query_attribute('title', request=request),
        'theater_email': theater.contact_email or '',
        'theater_phone': theater.phone_number or '',
        'theater_logo': '',
        'sender_name': request.principal.title,
        'sender_email': '',
        'booking_date': format_date(IZopeDublinCore(session).created),
        'session': get_object_label(session, request, view, name='short-text'),
        'session_title': get_object_label(entry, request, view),
        'session_date': format_datetime(session.start_date)
    }
    logo = theater.logo
    if logo:
        thumbnail = IThumbnails(logo).get_thumbnail('h160')
        values['theater_logo'] = (f'<img src="{absolute_url(thumbnail, request)}"'
                                  f' alt="Logo" />')
    mail_info = IPrincipalMailInfo(request.principal, None)
    if mail_info is not None:
        values['sender_email'] = next(mail_info.get_addresses())
    return values


def get_booking_message(subject, data, context, request, settings):
    """Booking notification message getter"""
    sm = get_utility(ISecurityManager)
    principal = sm.get_raw_principal(context.recipient)
    mail_info = IPrincipalMailInfo(principal, None)
    if mail_info is None:
        return
    mail_addresses = [
        f'{name} <{address}>'
        for name, address in mail_info.get_addresses()
    ]
    translate = request.localizer.translate
    subject = data.get('notify_subject') or translate(subject)
    cc = None
    theater = get_parent(context, IMovieTheater)
    templates = IMailTemplates(theater)
    if templates.send_copy_to_sender:
        principal = request.principal
        if principal is not None:
            mail_info = IPrincipalMailInfo(principal, None)
            if mail_info is not None:
                cc = [
                    f'{name} <{address}>'
                    for name, address in mail_info.get_addresses()
                ]
    message = HTMLMessage(f'{settings.subject_prefix} {subject}',
                          from_addr=f'{settings.source_name} <{settings.source_address}>',
                          to_addr=mail_addresses,
                          cc=cc,
                          html=data.get('notify_message'))
    if data.get('include_quotation'):
        quotation = context.get_quotation()
        if quotation is not None:
            message.attach(Attachment(content_type='application/pdf',
                                      data=quotation.data,
                                      filename=f'{context.quotation_number}.pdf'))
    return message



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.schema import Bool, Float, Int, Text, TextLine

from pyams_app_msc import _


PRICES_VOCABULARY = 'msc.theater.prices'


class ICinemaPrice(IAttributeAnnotatable):
    """Cinema price interface"""

    active = Bool(title=_("Active price?"),
                  description=_("An inactive price can't be assigned to new reservations"),
                  required=True,
                  default=True)

    name = TextLine(title=_("Price name"),
                    description=_("This name should be unique for each theater price..."),
                    required=True)

    participant_price = Float(title=_("Participant price"),
                              description=_("This is the price which is applied to each participant"),
                              required=True)

    accompanying_ratio = Int(title=_("Accompanying ratio"),
                             description=_("You can set an integer number which will define the count of "
                                           "participants for which one accompanying person will have free access; "
                                           "this ratio will still be updatable on all bookings using this price"),
                             default=0,
                             min=0,
                             required=False)

    accompanying_price = Float(title=_("Accompanying price"),
                               description=_("This is the price which is applied to each accompanying person"),
                               required=True)

    comment = Text(title=_("Comments"),
                   description=_("Additional comments"),
                   required=False)

    notepad = Text(title=_("Notepad"),
                   description=_("This comment is for internal use only"),
                   required=False)


CINEMA_PRICE_CONTAINER_KEY = 'msc.price.container'


class ICinemaPriceContainer(IContainer):
    """Cinema price container interface"""

    contains(ICinemaPrice)

    def append(self, item):
        """Add price to container"""

    def get_active_items(self):
        """Get iterator over active items"""


class ICinemaPriceContainerTarget(IAttributeAnnotatable):
    """Cinema price container target marker interface"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_app_msc.interfaces import MSC_SITES_MANAGER_ROLE
from pyams_security.schema import PrincipalsSetField

from pyams_app_msc import _


MSC_SITEROOT_ROLES = 'msc.root.roles'


class IMSCSiteRootRoles(Interface):
    """Site root roles interface"""

    msc_site_managers = PrincipalsSetField(title=_("Sites managers"),
                                           description=_("These principals are allowed to create "
                                                         "and manage sites and movie theaters"),
                                           role_id=MSC_SITES_MANAGER_ROLE,
                                           required=False)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_psc.feature.profile.task module

This module provides a custom scheduler task which is used to remove
user profiles which have not been activated.
"""

import logging
from datetime import datetime, timedelta, timezone

from pyramid.events import subscriber
from transaction.interfaces import ITransactionManager
from zope.interface import implementer
from zope.principalannotation.interfaces import IPrincipalAnnotationUtility

from pyams_app_msc.feature.profile.interfaces import IActivatedPrincipalEvent, IRegisteredPrincipalEvent, \
    IUserProfileCleanerTask
from pyams_scheduler.interfaces import IScheduler, ISchedulerProcess, SCHEDULER_NAME
from pyams_scheduler.interfaces.task import IDateTaskScheduling, SCHEDULER_TASK_DATE_MODE, TASK_STATUS_OK
from pyams_scheduler.process import TaskResettingThread
from pyams_scheduler.task import Task
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import IUnknownPrincipalInfo
from pyams_security.interfaces.names import INTERNAL_USER_ID
from pyams_security_views.interfaces.login import ILoginConfiguration
from pyams_site.interfaces import ISiteRoot, PYAMS_APPLICATION_DEFAULT_NAME, PYAMS_APPLICATION_SETTINGS_KEY
from pyams_utils.factory import create_object, factory_config
from pyams_utils.registry import get_pyramid_registry, get_utility, query_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.zodb import ZODBConnection
from pyams_zmq.interfaces import IZMQProcessStartedEvent

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


LOGGER = logging.getLogger('PyAMS (msc)')


@factory_config(IUserProfileCleanerTask)
class UserProfileCleanerTask(Task):
    """User profile cleaner task"""

    label = _("User profile cleaner")
    icon_class = 'fas fa-user'

    principal_id = INTERNAL_USER_ID

    def __init__(self, user_id):
        super().__init__()
        self.user_id = user_id

    def run(self, report, **kwargs):
        """Run user profile cleaner task"""
        # check principal annotations
        utility = get_utility(IPrincipalAnnotationUtility)
        if self.user_id in utility.annotations:
            del utility.annotations[self.user_id]
        # check user profile
        sm = get_utility(ISecurityManager)
        principal = sm.get_raw_principal(self.user_id)
        if principal is not None:
            del principal.__parent__[principal.__name__]
        # remove task after execution
        if self.__parent__ is not None:
            del self.__parent__[self.__name__]
        return TASK_STATUS_OK, None


@subscriber(IZMQProcessStartedEvent, context_selector=ISchedulerProcess)
def handle_scheduler_start(event):
    """Check for profiles cleaner tasks

    Profile cleaner tasks are typically automatically deleted after their execution.
    If tasks with passed execution date are still present in the scheduler, this is generally
    because scheduler was stopped at task execution time; so tasks which where not run are
    re-scheduled at process startup in a very near future...
    """
    with ZODBConnection() as root:
        registry = get_pyramid_registry()
        application_name = registry.settings.get(PYAMS_APPLICATION_SETTINGS_KEY,
                                                 PYAMS_APPLICATION_DEFAULT_NAME)
        application = root.get(application_name)
        sm = application.getSiteManager()  # pylint: disable=invalid-name
        scheduler = sm.get(SCHEDULER_NAME)
        if scheduler is not None:
            LOGGER.debug("Checking pending scheduler tasks on {!r}".format(scheduler))
            for task in scheduler.values():
                if not IUserProfileCleanerTask.providedBy(task):
                    continue
                schedule_info = IDateTaskScheduling(task, None)
                if schedule_info is None:  # no date scheduling
                    continue
                now = datetime.now(timezone.utc)
                if schedule_info.active and (schedule_info.start_date < now):
                    # we add a small amount of time to be sure that scheduler and indexer
                    # processes are started...
                    schedule_info.start_date = now + timedelta(minutes=1)
                    # commit update for reset thread to get updated data!!
                    ITransactionManager(task).commit()
                    # start task resetting thread
                    LOGGER.debug(" - restarting task « {} »".format(task.name))
                    TaskResettingThread(event.object, task).start()


@subscriber(IRegisteredPrincipalEvent)
def handle_registered_profile(event):
    """Handle registered profile event"""
    scheduler = query_utility(IScheduler)
    if scheduler is None:
        return
    sm = get_utility(ISecurityManager)
    principal_id = event.object.id
    principal = sm.get_raw_principal(principal_id)
    if IUnknownPrincipalInfo.providedBy(principal):
        return
    task_id = f'profile_cleaner::{principal_id}'
    if task_id in scheduler:
        del scheduler[task_id]
    root = get_parent(scheduler, ISiteRoot)
    configuration = ILoginConfiguration(root)
    task = create_object(IUserProfileCleanerTask, user_id=principal_id)
    task.name = f'Profile cleaner: {principal_id}'
    task.schedule_mode = SCHEDULER_TASK_DATE_MODE
    scheduler_info = IDateTaskScheduling(task)
    scheduler_info.start_date = (tztime(datetime.now(timezone.utc)) +
                                 timedelta(days=configuration.activation_delay))
    scheduler_info.active = True
    scheduler[task_id] = task


@subscriber(IActivatedPrincipalEvent)
def handle_activated_profile(event):
    """Handle activated profile event"""
    scheduler = query_utility(IScheduler)
    if scheduler is None:
        return
    principal_id = event.object.id
    task_id = f'profile_cleaner::{principal_id}'
    if task_id in scheduler:
        del scheduler[task_id]



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.shared.catalog.interfaces import IAudienceFilter, IDurationFilter
from pyams_content.feature.filter.interfaces import IFiltersContainer
from pyams_content.feature.filter.zmi import FilterAddForm, FilterAddMenu, FilterEditForm, FiltersTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import MANAGE_TEMPLATE_PERMISSION
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


#
# Activity audiences filter components
#

@viewlet_config(name='add-audience-filter.menu',
                context=IFiltersContainer, layer=IAdminLayer, view=FiltersTable,
                manager=IContextAddingsViewletManager, weight=100,
                permission=MANAGE_TEMPLATE_PERMISSION)
class AudienceFilterAddMenu(FilterAddMenu):
    """Audience filter add menu"""

    label = _("Audience filter")
    href = 'add-audience-filter.html'


@ajax_form_config(name='add-audience-filter.html',
                  context=IFiltersContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class AudienceFilterAddForm(FilterAddForm):
    """Audience filter add form"""

    content_factory = IAudienceFilter


@ajax_form_config(name='properties.html',
                  context=IAudienceFilter, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class AudienceFilterEditForm(FilterEditForm):
    """Audience filter edit form"""


#
# Activity duration filter components
#

@viewlet_config(name='add-duration-filter.menu',
                context=IFiltersContainer, layer=IAdminLayer, view=FiltersTable,
                manager=IContextAddingsViewletManager, weight=110,
                permission=MANAGE_TEMPLATE_PERMISSION)
class DurationFilterAddMenu(FilterAddMenu):
    """Duration filter add menu"""

    label = _("Duration filter")
    href = 'add-duration-filter.html'


@ajax_form_config(name='add-duration-filter.html',
                  context=IFiltersContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class DurationFilterAddForm(FilterAddForm):
    """Duration filter add form"""

    fields = Fields(IDurationFilter).omit('visible')
    content_factory = IDurationFilter


@ajax_form_config(name='properties.html',
                  context=IDurationFilter, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class DurationFilterEditForm(FilterEditForm):
    """Duration filter edit form"""

    fields = Fields(IDurationFilter).omit('visible')



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPForbidden, HTTPNotFound
from zope.interface import Interface
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema import Bool, Text, TextLine

from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS, IBookingContainer, IBookingInfo
from pyams_app_msc.feature.planning.interfaces import IPlanning, IPlanningTarget, ISession, VERSION_INFO_VOCABULARY
from pyams_app_msc.interfaces import CANCEL_BOOKING_PERMISSION, CREATE_BOOKING_PERMISSION, MSC_MANAGER_ROLE, \
    MSC_OPERATOR_ROLE
from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.audience import ICinemaAudienceContainer
from pyams_content.feature.history.interfaces import IHistoryContainer
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import AddForm
from pyams_form.interfaces import DISPLAY_MODE, HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import IProtectedObject
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.schema.button import SubmitButton
from pyams_template.template import template_config
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url, canonical_url
from pyams_utils.zodb import load_object
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyams_zmi.utils import get_object_label

try:
    from pyams_chat.interfaces import IChatMessage, IChatMessageHandler
    from pyams_chat.message import ChatMessage
except ImportError:
    ChatMessage = None


__docformat__ = 'restructuredtext'

from pyams_app_msc import _


#
# Booking add form
#

class IBookingAddFormInfo(IBookingInfo):
    """Booking add form interface"""

    session_id = TextLine(title=_("Session ID"),
                          required=True)

    comments = Text(title=_("Comments"),
                    description=_("You can add optional comments to your booking request"),
                    required=False)

    send_confirmation = Bool(title=_("Get confirmation message?"),
                             description=_("If 'yes', a confirmation message will be sent to you to "
                                           "acknowledge the booking"),
                             required=True,
                             default=True)


class IBookingAddFormButtons(Interface):
    """Booking add form buttons interface"""

    add = SubmitButton(name='add',
                       title=_("Add booking for this session"))


@ajax_form_config(name='booking-new.html',
                  context=IPlanningTarget, layer=IPyAMSLayer,
                  permission=CREATE_BOOKING_PERMISSION)
class BookingAddForm(AddForm, PortalContextIndexPage):
    """Booking add form"""

    legend = _("Your new booking properties")

    fields = Fields(IBookingAddFormInfo).select('session_id', 'nb_participants',
                                                'nb_accompanists', 'nb_groups', 'cultural_pass',
                                                'comments', 'send_confirmation')
    buttons = Buttons(IBookingAddFormButtons)

    content_factory = IBookingInfo
    _edit_permission = CREATE_BOOKING_PERMISSION

    def get_ajax_handler(self):
        return canonical_url(self.context, self.request, self.ajax_form_handler)

    @reify
    def session(self):
        """Session getter"""
        session_id_widget = self.widgets.get('session_id')
        if session_id_widget is not None:
            session_id = session_id_widget.value
        else:
            session_id = self.request.params.get('session_id')
        if session_id:
            planning = IPlanning(self.context)
            return planning.get(session_id)
        return None

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        session_id = self.widgets.get('session_id')
        if session_id is not None:
            session_id.mode = HIDDEN_MODE
            if 'session_id' in self.request.params:
                session_id.value = self.request.params['session_id']

    @handler(buttons['add'])
    def handle_add(self, action):
        super().handle_add(self, action)

    def update_content(self, obj, data):
        data = data.get(self, data)
        for name in ('nb_participants', 'nb_accompanists', 'nb_groups', 'cultural_pass',
                     'comments', 'send_confirmation'):
            setattr(obj, name, data.get(name))
        obj.recipient = self.request.principal.id
        obj.creator = self.request.principal.id

    def add(self, obj):
        del obj.session
        container = IBookingContainer(self.session, None)
        if container is not None:
            container.append(obj)


@viewlet_config(name='booking-new.header',
                context=IPlanningTarget, layer=IPyAMSLayer, view=BookingAddForm,
                manager=IFormHeaderViewletManager, weight=10)
@template_config(template='templates/booking-new-header.pt',
                 layer=IPyAMSLayer)
class NewBookingHeader(Viewlet):
    """New booking header viewlet"""

    @property
    def catalog_entry(self):
        return get_parent(self.context, IWfCatalogEntry)

    @property
    def session(self):
        return self.view.session

    @property
    def start_date(self):
        start_date = self.view.session.start_date
        return format_datetime(start_date)

    def get_version(self):
        version = VERSION_INFO_VOCABULARY.by_value.get(self.view.session.version)
        if not version:
            return None
        return self.request.localizer.translate(version.title)

    def get_audiences(self):
        theater = get_parent(self.context, IMovieTheater)
        audiences = ICinemaAudienceContainer(theater)
        return ', '.join(map(lambda x: x.name,
                             filter(lambda x: x is not None,
                                    (
                                        audiences.get(audience)
                                        for audience in self.session.audiences
                                    ))))


@adapter_config(required=(IPlanningTarget, IPyAMSLayer, BookingAddForm),
                provides=IAJAXFormRenderer)
class BookingAddFormRenderer(ContextRequestViewAdapter):
    """Booking add form renderer"""

    def render(self, changes):
        if changes is None:
            return
        if ChatMessage is not None:
            request = self.request
            session = self.view.session
            booking = self.view.finished_state.get('changes')
            translate = request.localizer.translate
            message = ChatMessage(request=request,
                                  context=session,
                                  action='notify',
                                  category='booking.created',
                                  source=request.principal,
                                  title=translate(_("New booking request")),
                                  message=translate(_("{principal}: {session}")).format(
                                      principal=request.principal.title,
                                      session=get_object_label(session, request, self.view, name='text')),
                                  url=absolute_url(session, request, 'bookings.html'),
                                  modal=True,
                                  comment=booking.comments)
            message.send()
        return {
            'status': 'redirect',
            'location': canonical_url(self.context, self.request, 'booking-ok.html')
        }


@pagelet_config(name='booking-ok.html',
                context=IPlanningTarget, layer=IPyAMSLayer)
@template_config(template='templates/booking-ok.pt',
                 layer=IPyAMSLayer)
class BookingOKView(PortalContextIndexPage):
    """Booking acknowledge view"""


#
# Booking cancel form
#

class IBookingCancelFormInfo(Interface):
    """Booking cancel form interface"""

    booking_id = TextLine(title=_("Booking ID"),
                          required=True)

    session_label = TextLine(title=_("Session"),
                             required=False,
                             readonly=True)

    comments = Text(title=_("Comments"),
                    description=_("Please add optional comments to cancel your booking"),
                    required=True)


class IBookingCancelFormButtons(Interface):
    """Booking cancel form buttons"""

    cancel = SubmitButton(name='cancel',
                          title=_("Cancel booking"))


@ajax_form_config(name='cancel-booking.html',
                  context=ISiteRoot, layer=IPyAMSLayer)
class BookingCancelForm(AddForm, PortalContextIndexPage):
    """Booking cancel form"""

    legend = _("Cancel booking")

    fields = Fields(IBookingCancelFormInfo)
    buttons = Buttons(IBookingCancelFormButtons)

    content_factory = None
    _edit_permission = CANCEL_BOOKING_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        content = self.get_content()
        session = ISession(content)
        booking_id = self.widgets.get('booking_id')
        if booking_id is not None:
            booking_id.value = ICacheKeyValue(content)
            booking_id.mode = HIDDEN_MODE
        session_label = self.widgets.get('session_label')
        if session_label is not None:
            session_label.value = get_object_label(session, self.request, self, 'text')
            session_label.mode = DISPLAY_MODE

    @handler(buttons['cancel'])
    def handle_cancel(self, action):
        super().handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        booking = self.get_content()
        request = self.request
        comments = data.get('comments')
        history = IHistoryContainer(booking, None)
        if history is not None:
            history.add_history(booking,
                                comment=comments,
                                request=request)
        booking.status = BOOKING_STATUS.CANCELLED.value
        request.registry.notify(ObjectModifiedEvent(booking))
        return booking


@adapter_config(required=(ISiteRoot, IPyAMSLayer, BookingCancelForm),
                provides=IFormContent)
def booking_cancel_form_content(context, request, form):
    """Booking cancel form content getter"""
    booking_id = request.params.get('booking_id')
    if not booking_id:
        booking_id = request.params.get('form.widgets.booking_id')
    if not booking_id:
        raise HTTPNotFound()
    booking = load_object(booking_id, context)
    if not IBookingInfo.providedBy(booking):
        raise HTTPNotFound()
    if booking.recipient != request.principal.id:
        raise HTTPForbidden()
    return booking


@adapter_config(required=(ISiteRoot, IPyAMSLayer, BookingCancelForm),
                provides=IAJAXFormRenderer)
class BookingCancelFormRenderer(ContextRequestViewAdapter):
    """Booking cancel form renderer"""

    def render(self, changes):
        if changes is None:
            return None
        if ChatMessage is not None:
            request = self.request
            booking = IBookingInfo(changes)
            session = get_parent(booking, ISession)
            translate = request.localizer.translate
            history = IHistoryContainer(booking)
            message = ChatMessage(request=request,
                                  context=session,
                                  action='notify',
                                  category='booking.cancelled',
                                  source=request.principal,
                                  title=translate(_("Cancelled booking request")),
                                  message=translate(_("{principal}: {session}")).format(
                                      principal=request.principal.title,
                                      session=get_object_label(session, request, self.view, 'text')),
                                  url=absolute_url(session, request, 'bookings.html'),
                                  modal=True,
                                  comment=list(history.values())[-1].comment)
            message.send()
        return {
            'status': 'redirect',
            'location': absolute_url(self.context, self.request, 'my-dashboard.html')
        }


if ChatMessage is not None:

    @adapter_config(name='booking.created',
                    required=IChatMessage,
                    provides=IChatMessageHandler)
    @adapter_config(name='booking.cancelled',
                    required=IChatMessage,
                    provides=IChatMessageHandler)
    class BookingChatMessageHandler(ContextAdapter):
        """Booking chat message handler"""

        def get_target(self):
            """Chat message targets getter"""
            theater = get_parent(self.context.context, IMovieTheater)
            protection = IProtectedObject(theater)
            principals = set()
            for role_id in (MSC_MANAGER_ROLE, MSC_OPERATOR_ROLE):
                for principal_id in protection.get_principals(role_id):
                    principals.add(principal_id)
            return {
                'principals': tuple(principals)
            }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from datetime import date
from io import BytesIO

from PIL import Image, ImageStat
from persistent import Persistent
from persistent.mapping import PersistentMapping
from zope.container.contained import Contained
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION
from pyams_app_msc.shared.theater import IMovieTheater, IMovieTheaterSettings, MOVIE_THEATER_SETTINGS_KEY
from pyams_file.property import FileProperty
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.request import query_request


@factory_config(IMovieTheaterSettings)
class MovieTheaterSettings(Persistent, Contained):
    """Movie theater settings persistent class"""

    calendar_first_day = FieldProperty(IMovieTheaterSettings['calendar_first_day'])
    calendar_slot_duration = FieldProperty(IMovieTheaterSettings['calendar_slot_duration'])
    default_session_duration = FieldProperty(IMovieTheaterSettings['default_session_duration'])
    session_duration_delta = FieldProperty(IMovieTheaterSettings['session_duration_delta'])

    allow_session_request = FieldProperty(IMovieTheaterSettings['allow_session_request'])
    session_request_mode = FieldProperty(IMovieTheaterSettings['session_request_mode'])

    reminder_delay = FieldProperty(IMovieTheaterSettings['reminder_delay'])

    booking_cancel_mode = FieldProperty(IMovieTheaterSettings['booking_cancel_mode'])
    booking_cancel_max_delay = FieldProperty(IMovieTheaterSettings['booking_cancel_max_delay'])
    booking_cancel_notice_period = FieldProperty(IMovieTheaterSettings['booking_cancel_notice_period'])
    booking_retention_duration = FieldProperty(IMovieTheaterSettings['booking_retention_duration'])

    quotation_number_format = FieldProperty(IMovieTheaterSettings['quotation_number_format'])
    quotation_logo = FileProperty(IMovieTheaterSettings['quotation_logo'])
    quotation_color = FieldProperty(IMovieTheaterSettings['quotation_color'])
    currency = FieldProperty(IMovieTheaterSettings['currency'])
    vat_rate = FieldProperty(IMovieTheaterSettings['vat_rate'])

    _year_quotations = None
    _month_quotations = None

    def get_logo_color(self):
        """Extract median color from logo"""
        logo = self.quotation_logo
        if not logo:
            return None
        image = Image.open(BytesIO(logo.data))
        median = ImageStat.Stat(image).median
        return "{:02x}{:02x}{:02x}".format(*median)

    def get_quotation_number(self):
        """Get new quotation number"""
        request = query_request()
        principal = request.principal
        initials = ''.join(map(lambda x: x[0].upper(), principal.title.split()))
        now = date.today()
        year_quotations = self._year_quotations
        if year_quotations is None:
            year_quotations = self._year_quotations = PersistentMapping()
        year_inc = year_quotations.get(now.year, 0) + 1
        year_quotations[now.year] = year_inc
        month_quotations = self._month_quotations
        if month_quotations is None:
            month_quotations = self._month_quotations = PersistentMapping()
        month_inc = month_quotations.get(now.year, {}).get(now.month, 0) + 1
        month_quotations.setdefault(now.year, PersistentMapping())[now.month] = month_inc
        args = {
            'yyyy': now.year,
            'yy': int(str(now.year)[2:]),
            'mm': now.month,
            'dd': now.day,
            'yinc': year_inc,
            'minc': month_inc,
            'operator': initials
        }
        return self.quotation_number_format.format(**args)

    def get_quotation_color(self):
        return f'#{self.quotation_color or self.get_logo_color()}'


@adapter_config(required=IMovieTheaterSettings,
                provides=IViewContextPermissionChecker)
class MovieTheaterSettingsPermissionChecker(ContextAdapter):
    """Movie theater settings permission checker"""

    edit_permission = MANAGE_THEATER_PERMISSION


@adapter_config(required=IMovieTheater,
                provides=IMovieTheaterSettings)
def movie_theater_settings(context):
    """Movie theater settings adapter"""
    return get_annotation_adapter(context, MOVIE_THEATER_SETTINGS_KEY, IMovieTheaterSettings,
                                  name='++settings++')


@adapter_config(name='settings',
                required=IMovieTheater,
                provides=ITraversable)
class MovieTheaterSettingsTraverser(ContextAdapter):
    """Movie theater settings traverser"""

    def traverse(self, name, furtherpath=None):
        return IMovieTheaterSettings(self.context, None)


@adapter_config(name='settings',
                required=IMovieTheater,
                provides=ISublocations)
class MovieTheaterSettingsSublocations(ContextAdapter):
    """Movie theater settings sub-locations adapter"""

    def sublocations(self):
        settings = IMovieTheaterSettings(self.context, None)
        if settings is not None:
            yield settings



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from fanstatic import Library, Resource
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IControlPanelMenu, INavigationViewletManager
from pyams_zmi.zmi.viewlet.menu import ControlPanelMenu


library = Library('msc', 'resources')

msc = Resource(library, 'js/msc.js',
               minified='js/msc.min.js')


@viewletmanager_config(name='control-panel.menu', layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=300,
                       provides=IControlPanelMenu,
                       permission=MANAGE_SYSTEM_PERMISSION)
class MSCControlPanelMenu(ControlPanelMenu):
    """MSC control panel menu"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Or
from pyramid.events import subscriber

from pyams_app_msc.feature.profile import IUserProfile
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_catalog.query import CatalogResultSet
from pyams_security.interfaces import LOGIN_REFERER_KEY
from pyams_security.utility import get_principal
from pyams_utils.factory import get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_utils.url import absolute_url

__docformat__ = 'restructuredtext'


try:

    from pyams_auth_oauth.interfaces import IOAuthAuthenticationEvent

    @subscriber(IOAuthAuthenticationEvent)
    def handle_oauth_authentication(event):
        """Handle redirect on OAuth authentication"""
        principal_id = event.principal_id
        principal = get_principal(principal_id=principal_id)
        profile_info = IUserProfile(principal, None)
        if profile_info is None:
            return
        request = event.request
        if not (profile_info.firstname and profile_info.lastname and
                profile_info.email and profile_info.establishment and
                profile_info.local_theaters):
            event.redirect_location = absolute_url(request.root, request, 'my-profile.html')
        else:
            theaters_names = profile_info.local_theaters
            if len(theaters_names) == 1:
                theater = get_utility(IMovieTheater, name=theaters_names[0])
                if theater is not None:
                    event.redirect_location = absolute_url(theater, request)
                    return
            session = request.session
            if LOGIN_REFERER_KEY in session:
                event.redirect_location = session[LOGIN_REFERER_KEY]
                del session[LOGIN_REFERER_KEY]
            else:
                catalog = get_utility(ICatalog)
                query = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                            Or(Eq(catalog['role:msc:manager'], principal_id),
                               Eq(catalog['role:msc:operator'], principal_id),
                               Eq(catalog['role:msc:contributor'], principal_id),
                               Eq(catalog['role:msc:reader'], principal_id)))
                theaters = list(CatalogResultSet(CatalogQuery(catalog).query(query)))
                if len(theaters) == 1:
                    theater = theaters[0]
                    event.redirect_location = absolute_url(theater, request, 'admin')
                elif len(theaters) > 1:
                    event.redirect_location = absolute_url(request.root, request,
                                                          'select-admin-theater.html')
                else:
                    event.redirect_location = absolute_url(request.context, request)

except ImportError:
    pass



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_app_msc.component.contact import IContact
from pyams_form.interfaces import IObjectFactory
from pyams_form.interfaces.form import IForm
from pyams_form.interfaces.widget import IObjectWidget
from pyams_layer.interfaces import IFormLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import get_interface_name, get_object_factory


@adapter_config(name=get_interface_name(IContact),
                required=(Interface, IFormLayer, IForm, IObjectWidget),
                provides=IObjectFactory)
def contact_factory(*args):  # pylint: disable=unused-argument
    """Address object factory"""
    return get_object_factory(IContact)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.feature.booking.interfaces import IBookingReminderTask
from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_scheduler.interfaces import MANAGE_TASKS_PERMISSION
from pyams_scheduler.task.zmi import BaseTaskEditForm

__docformat__ = 'restructuredtext'


@ajax_form_config(name='properties.html',
                  context=IBookingReminderTask, layer=IPyAMSLayer,
                  permission=MANAGE_TASKS_PERMISSION)
class BookingReminderTaskEditForm(BaseTaskEditForm):
    """Booking reminder task edit form"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.events import subscriber
from zope.container.ordered import OrderedContainer
from zope.interface import implementer
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectMovedEvent, IObjectRemovedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION
from pyams_app_msc.shared.catalog.interfaces import CATALOG_ENTRY_CONTENT_TYPE, ICatalogEntryInfo, ICatalogManagerTarget
from pyams_app_msc.shared.theater.interfaces import IMovieTheater, IMovieTheaterRoles, IMovieTheaterSettings, \
    MOVIE_THEATER_ROLES, MOVIE_THEATER_SETTINGS_KEY, MSC_THEATERS_VOCABULARY
from pyams_app_msc.shared.theater.interfaces.audience import ICinemaAudienceContainerTarget
from pyams_app_msc.shared.theater.interfaces.mail import IMailTemplatesTarget
from pyams_app_msc.shared.theater.interfaces.price import ICinemaPriceContainerTarget
from pyams_app_msc.shared.theater.interfaces.room import ICinemaRoomContainer, ICinemaRoomContainerTarget
from pyams_content.component.illustration.interfaces import IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphFactorySettingsTarget
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.interfaces import IObjectType
from pyams_content.shared.common.interfaces import CONTENT_MANAGER_ROLES, ISharedContent
from pyams_content.shared.common.manager import BaseSharedTool
from pyams_content.shared.common.types import TypedSharedToolMixin
from pyams_file.property import FileProperty
from pyams_i18n.interfaces import II18n
from pyams_layer.skin import UserSkinnableContentMixin
from pyams_portal.interfaces import IPortalContext, IPortalFooterContext, IPortalHeaderContext
from pyams_security.interfaces import IDefaultProtectionPolicy, IRolesPolicy, IViewContextPermissionChecker
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObjectMixin, ProtectedObjectRoles
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextAdapter, NullAdapter, adapter_config
from pyams_utils.factory import factory_config, get_interface_base_name, get_object_factory
from pyams_utils.registry import get_utilities_for
from pyams_utils.request import check_request, query_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@factory_config(IMovieTheater)
@implementer(IDefaultProtectionPolicy, IIllustrationTarget, ILinkIllustrationTarget,
             ICinemaRoomContainerTarget, ICinemaPriceContainerTarget, ICinemaAudienceContainerTarget,
             ICatalogManagerTarget, IMailTemplatesTarget, IParagraphFactorySettingsTarget,
             IPortalContext, IPortalHeaderContext, IPortalFooterContext, IPreviewTarget)
class Theater(OrderedContainer, BaseSharedTool, TypedSharedToolMixin,
              ProtectedObjectMixin, UserSkinnableContentMixin):
    """Main theater persistent class"""

    title = FieldProperty(IMovieTheater['title'])
    short_name = FieldProperty(IMovieTheater['short_name'])
    header = FieldProperty(IMovieTheater['header'])
    description = FieldProperty(IMovieTheater['description'])
    logo = FileProperty(IMovieTheater['logo'])
    address = FieldProperty(IMovieTheater['address'])
    web_address = FieldProperty(IMovieTheater['web_address'])
    contact_email = FieldProperty(IMovieTheater['contact_email'])
    phone_number = FieldProperty(IMovieTheater['phone_number'])
    banking_account = FieldProperty(IMovieTheater['banking_account'])
    admin_info = FieldProperty(IMovieTheater['admin_info'])
    notepad = FieldProperty(IMovieTheater['notepad'])

    sequence_name = ''
    sequence_prefix = ''

    content_name = _("Theater")
    shared_content_menu = False
    shared_content_type = CATALOG_ENTRY_CONTENT_TYPE
    shared_content_info_factory = ICatalogEntryInfo

    @property
    def shared_content_factory(self):
        return get_object_factory(ISharedContent, name=self.shared_content_type)

    def is_deletable(self):
        """Check if theater can be deleted"""
        container = ICinemaRoomContainer(self)
        return len(container.keys()) == 0


@adapter_config(required=IMovieTheater,
                provides=IObjectType)
def movie_theater_object_type(context):
    """Movie theater object type adapter"""
    return get_interface_base_name(IMovieTheater)


@implementer(IMovieTheaterRoles)
class MovieTheaterRoles(ProtectedObjectRoles):
    """Movie theater roles"""

    msc_managers = RolePrincipalsFieldProperty(IMovieTheaterRoles['msc_managers'])
    msc_operators = RolePrincipalsFieldProperty(IMovieTheaterRoles['msc_operators'])
    msc_contributors = RolePrincipalsFieldProperty(IMovieTheaterRoles['msc_contributors'])
    msc_designers = RolePrincipalsFieldProperty(IMovieTheaterRoles['msc_designers'])
    msc_readers = RolePrincipalsFieldProperty(IMovieTheaterRoles['msc_readers'])


@adapter_config(required=IMovieTheater,
                provides=IMovieTheaterRoles)
def movie_theater_roles(context):
    """Movie theater roles adapter"""
    return MovieTheaterRoles(context)


@adapter_config(name=CONTENT_MANAGER_ROLES,
                required=IMovieTheater,
                provides=IRolesPolicy)
class MovieTheaterContentManagerRolesPolicy(NullAdapter):
    """Movie theater content manager roles policy"""


@adapter_config(name=MOVIE_THEATER_ROLES,
                required=IMovieTheater,
                provides=IRolesPolicy)
class MovieTheaterRolesPolicy(ContextAdapter):
    """Movie theater roles policy"""

    roles_interface = IMovieTheaterRoles
    weight = 20


@subscriber(IObjectAddedEvent, context_selector=IMovieTheater)
def handle_added_movie_theater(event):
    """Register movie theater when added"""
    site = get_parent(event.newParent, ISiteRoot)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IMovieTheater, name=event.newName)


@subscriber(IObjectMovedEvent, context_selector=IMovieTheater)
def handle_moved_movie_theater(event: IObjectMovedEvent):
    """Update movie theater registration when renamed"""
    if IObjectRemovedEvent.providedBy(event):
        return
    request = check_request()
    registry = request.root.getSiteManager()
    if registry is not None:
        old_name = event.oldName
        new_name = event.newName
        if old_name == new_name:
            return
        registry.unregisterUtility(event.object, IMovieTheater, name=old_name)
        if new_name:
            registry.registerUtility(event.object, IMovieTheater, name=new_name)


@subscriber(IObjectRemovedEvent, context_selector=IMovieTheater)
def handle_deleted_movie_theater(event: IObjectRemovedEvent):
    """Un-register movie theater when deleted"""
    site = get_parent(event.oldParent, ISiteRoot)
    registry = site.getSiteManager()
    if registry is not None:
        registry.unregisterUtility(event.object, IMovieTheater, name=event.oldName)


@adapter_config(required=IMovieTheater,
                provides=IViewContextPermissionChecker)
class MovieTheaterPermissionChecker(ContextAdapter):
    """Movie theater edit permission checker"""

    edit_permission = MANAGE_THEATER_PERMISSION


@vocabulary_config(name=MSC_THEATERS_VOCABULARY)
class MovieTheaterVocabulary(SimpleVocabulary):
    """Movie theaters vocabulary"""

    interface = IMovieTheater

    def __init__(self, context=None):
        request = query_request()
        super().__init__(sorted([
            SimpleTerm(v, title=II18n(t).query_attribute('title', request=request))
            for v, t in get_utilities_for(self.interface)
        ], key=lambda x: x.title))



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.shared.theater import IMovieTheater
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage

__docformat__ = 'restructuredtext'


@pagelet_config(name='movies.html',
                context=IMovieTheater, layer=IPyAMSLayer)
class MovieTheaterMoviesPage(PortalContextIndexPage):
    """Movie theater movies list page"""

    page_name = 'movies'



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.booking.reminder module

This module provides components used to handle booking reminders.
"""

import logging
from datetime import datetime, timedelta, timezone

from pyramid.events import subscriber
from transaction.interfaces import ITransactionManager

from pyams_app_msc.feature.booking.interfaces import IBookingReminderTask
from pyams_scheduler.interfaces import ISchedulerProcess, SCHEDULER_NAME
from pyams_scheduler.interfaces.task import IDateTaskScheduling, TASK_STATUS_OK
from pyams_scheduler.process import TaskResettingThread
from pyams_scheduler.task import Task
from pyams_security.interfaces.names import INTERNAL_USER_ID
from pyams_site.interfaces import PYAMS_APPLICATION_DEFAULT_NAME, PYAMS_APPLICATION_SETTINGS_KEY
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.registry import get_pyramid_registry
from pyams_utils.timezone import gmtime
from pyams_utils.zodb import ZODBConnection, load_object
from pyams_zmq.interfaces import IZMQProcessStartedEvent

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


LOGGER = logging.getLogger('PyAMS (msc)')


@factory_config(IBookingReminderTask)
class BookingReminderTask(Task):
    """Booking reminder task"""

    label = _("Booking reminder")
    icon_class = 'fas fa-envelope'

    principal_id = INTERNAL_USER_ID

    def __init__(self, booking):
        super().__init__()
        self.booking_oid = ICacheKeyValue(booking)

    def run(self, report, **kwargs):
        booking = load_object(self.booking_oid, self)
        if booking is None:
            LOGGER.debug(f">>> can't find booking with OID {self.booking_oid}")
        else:
            booking.send_reminder_message()
        # remove task after execution!
        if self.__parent__ is not None:
            del self.__parent__[self.__name__]
        return TASK_STATUS_OK, None


@subscriber(IZMQProcessStartedEvent, context_selector=ISchedulerProcess)
def handle_scheduler_start(event):
    """Check for scheduler reminder tasks

    Booking reminders tasks are typically automatically deleted after their execution.
    If tasks with passed execution date are still present in the scheduler, this is generally
    because scheduler was stopped at task execution time; so tasks which where not run are
    re-scheduled at process startup in a very near future...
    """
    with ZODBConnection() as root:
        registry = get_pyramid_registry()
        application_name = registry.settings.get(PYAMS_APPLICATION_SETTINGS_KEY,
                                                 PYAMS_APPLICATION_DEFAULT_NAME)
        application = root.get(application_name)
        sm = application.getSiteManager()  # pylint: disable=invalid-name
        scheduler = sm.get(SCHEDULER_NAME)
        if scheduler is not None:
            LOGGER.debug("Checking pending scheduler tasks on {!r}".format(scheduler))
            for task in scheduler.values():
                if not IBookingReminderTask.providedBy(task):
                    continue
                schedule_info = IDateTaskScheduling(task, None)
                if schedule_info is None:  # no date scheduling
                    continue
                now = gmtime(datetime.now(timezone.utc))
                if schedule_info.active and (schedule_info.start_date < now):
                    # we add a small amount of time to be sure that scheduler and indexer
                    # processes are started...
                    schedule_info.start_date = now + timedelta(minutes=1)
                    # commit update for reset thread to get updated data!!
                    ITransactionManager(task).commit()
                    # start task resetting thread
                    LOGGER.debug(" - restarting task « {} »".format(task.name))
                    TaskResettingThread(event.object, task).start()



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.events import subscriber
from zope.component.interfaces import ISite
from zope.lifecycleevent.interfaces import IObjectAddedEvent

from pyams_app_msc.reference.structure.interfaces import IStructureType, IStructureTypeTable, STRUCTURE_TYPES_VOCABULARY
from pyams_content.reference import ReferenceInfo, ReferenceTable, ReferencesVocabulary
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'


@factory_config(IStructureTypeTable)
class StructureTypeTable(ReferenceTable):
    """Structure type table"""


@subscriber(IObjectAddedEvent, context_selector=IStructureTypeTable)
def handle_added_structures_types_table(event):
    """Handle new structures types table"""
    site = get_parent(event.object, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IStructureTypeTable)


@factory_config(IStructureType)
class StructureType(ReferenceInfo):
    """Structure type persistent class"""


@vocabulary_config(name=STRUCTURE_TYPES_VOCABULARY)
class StructureTypesVocabulary(ReferencesVocabulary):
    """Structure types vocabulary"""

    table_interface = IStructureTypeTable



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Any, Eq, Ge, Le
from zope.interface import Interface
from zope.schema import Bool, Choice
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_app_msc.feature.booking import IBookingInfo
from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS, BOOKING_STATUS_VOCABULARY
from pyams_app_msc.feature.booking.zmi.dashboard import BookingStatusTable, get_booking_element
from pyams_app_msc.feature.booking.zmi.interfaces import IBookingDashboardMenu
from pyams_app_msc.interfaces import VIEW_BOOKING_PERMISSION
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.room import ROOMS_VOCABULARY
from pyams_catalog.query import CatalogResultSet
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormFields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.schema import PrincipalField
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_table.interfaces import IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import get_interface_base_name
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_utility
from pyams_utils.schema import DatesRangeField
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import EmptyViewlet, viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.search import SearchForm, SearchResultsView, SearchView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='advanced-booking-search.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IBookingDashboardMenu, weight=40,
                permission=VIEW_BOOKING_PERMISSION)
class BookingAdvancedSearchMenu(NavigationMenuItem):
    """Booking advanced search menu"""

    label = _("Advanced search")
    href = '#booking-advanced-search.html'


class IBookingAdvancedSearchQuery(Interface):
    """Booking advanced search query interface"""

    recipient = PrincipalField(title=_("Recipient"),
                               required=False)

    status = Choice(title=_("Status"),
                    vocabulary=BOOKING_STATUS_VOCABULARY,
                    required=False)

    session = DatesRangeField(title=_("Session date"),
                              required=False)

    created = DatesRangeField(title=_("Creation date"),
                              required=False)

    modified = DatesRangeField(title=_("Modification date"),
                               required=False)

    include_archives = Bool(title=_("Include archives"),
                            required=False,
                            default=False)


class BookingAdvancedSearchForm(SearchForm):
    """Booking advanced search form"""

    title = _("Bookings search form")

    ajax_form_handler = 'booking-advanced-search-results.html'
    _edit_permission = VIEW_BOOKING_PERMISSION


@adapter_config(required=(Interface, IAdminLayer, BookingAdvancedSearchForm),
                provides=IFormFields)
def booking_advanced_search_form_fields(context, request, form):
    """Booking advanced search form fields"""
    return Fields(IBookingAdvancedSearchQuery)


@pagelet_config(name='booking-advanced-search.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=VIEW_BOOKING_PERMISSION)
class BookingAdvancedSearchView(SearchView):
    """Booking advanced search view"""

    title = _("Bookings search form")
    header_label = _("Advanced search")
    search_form = BookingAdvancedSearchForm


class BookingAdvancedSearchResultsTable(BookingStatusTable):
    """Booking advanced search form results table"""


@adapter_config(required=(IMovieTheater, IPyAMSLayer, BookingAdvancedSearchResultsTable),
                provides=IValues)
class BookingAdvancedSearchResultsValues(ContextRequestViewAdapter):
    """Booking advanced search results values"""

    def get_params(self, data):
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, ROOMS_VOCABULARY)
        params = And(Eq(catalog['object_types'], get_interface_base_name(IBookingInfo)),
                     Any(catalog['planning_room'], vocabulary.by_value.keys()),
                     Eq(catalog['booking_status'], [status.value for status in BOOKING_STATUS]))
        if data.get('recipient'):
            params &= Eq(catalog['booking_recipient'], data['recipient'])
        if data.get('status'):
            params &= Eq(catalog['booking_status'], data['status'])
        session_after, session_before = data.get('session', (None, None))
        if session_after:
            params &= Ge(catalog['planning_start_date'],
                         tztime(datetime.fromisoformat(session_after.isoformat())))
        if session_before:
            params &= Le(catalog['planning_end_date'],
                         tztime(datetime.fromisoformat(session_before.isoformat())))
        created_after, created_before = data.get('created', (None, None))
        if created_after:
            params &= Ge(catalog['created_date'],
                         tztime(datetime.fromisoformat(created_after.isoformat())))
        if created_before:
            params &= Le(catalog['created_date'],
                         tztime(datetime.fromisoformat(created_before.isoformat())))
        modified_after, modified_before = data.get('modified', (None, None))
        if modified_after:
            params &= Ge(catalog['modified_date'],
                         tztime(datetime.fromisoformat(modified_after.isoformat())))
        if modified_before:
            params &= Le(catalog['modified_date'],
                         tztime(datetime.fromisoformat(modified_before.isoformat())))
        return params

    @property
    def values(self):
        """Booking advanced search results values getter"""

        def true_filter(item):
            return True

        def archives_filter(item):
            return not item.booking.archived

        form = BookingAdvancedSearchForm(self.context, self.request)
        form.update()
        data, _errors = form.extract_data()
        params = self.get_params(data)
        catalog = get_utility(ICatalog)
        if data.get('include_archives'):
            booking_filter = true_filter
        else:
            booking_filter = archives_filter
        yield from filter(booking_filter,
                          map(get_booking_element,
                              unique_iter(CatalogResultSet(CatalogQuery(catalog).query(params)))))


@pagelet_config(name='booking-advanced-search-results.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=VIEW_BOOKING_PERMISSION, xhr=True)
class BookingAdvancedSearchResultsView(SearchResultsView):
    """Booking advanced search results view"""

    table_label = _("Search results")
    table_class = BookingAdvancedSearchResultsTable


@viewlet_config(name='pyams.content_header',
                layer=IAdminLayer, view=BookingAdvancedSearchResultsView,
                manager=IHeaderViewletManager, weight=10)
class BookingAdvancedSearchResultsViewHeaderViewlet(EmptyViewlet):
    """Booking advanced search results view header viewlet"""

    def render(self):
        return '<h1 class="mt-3"></h1>'



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface, Invalid
from zope.schema import TextLine

from pyams_app_msc import _
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import AddForm
from pyams_form.interfaces import HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import PUBLIC_PERMISSION
from pyams_security.interfaces.plugin import IUserRegistrationConfirmationInfo
from pyams_security.interfaces.profile import IUserRegistrationViews
from pyams_security_views.interfaces.login import ILoginConfiguration
from pyams_skin.schema.button import ResetButton, SubmitButton
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility


#
# Password reset request
#

class IPasswordResetInfo(Interface):
    """Password reset information interface"""

    login = TextLine(title=_("User ID"),
                     description=_("Your user ID can be your email address or a custom login"),
                     required=True)


class IPasswordResetButtons(Interface):
    """Password reset buttons"""

    change = SubmitButton(name='change',
                          title=_("Request password reset"))

    reset = ResetButton(name='reset',
                        title=_("Reset"))


@ajax_form_config(name='password-reset.html',
                  layer=IPyAMSLayer)
class PasswordResetForm(AddForm, PortalContextIndexPage):
    """Password reset form"""

    title = _("Password reset")
    legend = _("Enter account")

    fields = Fields(IPasswordResetInfo)
    buttons = Buttons(IPasswordResetButtons)

    _edit_permission = PUBLIC_PERMISSION

    @handler(buttons['change'])
    def handle_change(self, action):
        self.handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        login = data.get('login')
        if not login:
            self.widgets.errors += (Invalid(_("Login is required!")),)
            return
        login_info = ILoginConfiguration(self.request.root)
        if not login_info.allow_password_reset:
            self.widgets.errors += (Invalid(_("Password reset is not allowed!")),)
            return
        sm = get_utility(ISecurityManager)
        users_folder = sm.get(login_info.users_folder)
        user = users_folder.get(login)
        if user is not None:
            user.generate_reset_hash(notify=True, request=self.request)
            return user


@adapter_config(required=(Interface, IPyAMSLayer, PasswordResetForm),
                provides=IAJAXFormRenderer)
class PasswordResetFormRenderer(ContextRequestViewAdapter):
    """Password reset form renderer"""

    def render(self, changes):
        """Form renderer"""
        register_views = self.request.registry.getMultiAdapter((self.context, self.request),
                                                               IUserRegistrationViews)
        return {
            'status': 'redirect',
            'location': register_views.password_reset_final_view
        }


@pagelet_config(name='password-final.html',
                layer=IPyAMSLayer)
@template_config(template='templates/password-final.pt',
                 layer=IPyAMSLayer)
class PasswordResetFinalView(PortalContextIndexPage):
    """Password reset final view"""


#
# Password change
#

class IPasswordChangeButtons(Interface):
    """Password change buttons"""

    change = SubmitButton(name='change',
                          title=_("Change password"))

    reset = ResetButton(name='reset',
                        title=_("Reset"))


@ajax_form_config(name='password-change.html',
                  layer=IPyAMSLayer)
class PasswordChangeForm(AddForm, PortalContextIndexPage):
    """Password change form"""

    title = _("Password change")
    legend = _("Enter account and new password")

    fields = Fields(IUserRegistrationConfirmationInfo)
    buttons = Buttons(IPasswordChangeButtons)

    _edit_permission = PUBLIC_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        hash = self.widgets.get('activation_hash')
        if hash is not None:
            hash.mode = HIDDEN_MODE
            hash.value = self.request.params.get('hash')

    @handler(buttons['change'])
    def handle_change(self, action):
        self.handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        login = data.get('login')
        if not login:
            self.widgets.errors += (Invalid(_("Login is required!")),)
            return
        login_info = ILoginConfiguration(self.request.root)
        if not login_info.allow_password_reset:
            self.widgets.errors += (Invalid(_("Password reset is not allowed!")),)
            return
        sm = get_utility(ISecurityManager)
        users_folder = sm.get(login_info.users_folder)
        user = users_folder.get(login)
        if user is None:
            self.widgets.errors += (Invalid(_("Can't change password with provided arguments!")),)
        else:
            hash = data.get('activation_hash')
            try:
                user.reset_password(hash, data.get('password'))
            except Invalid as exc:
                translate = self.request.localizer.translate
                self.widgets.errors += (Invalid(', '.join(map(translate, exc.args))),)
            else:
                return user


@adapter_config(required=(Interface, IPyAMSLayer, PasswordChangeForm),
                provides=IAJAXFormRenderer)
class PasswordChangeFormRenderer(ContextRequestViewAdapter):
    """Password change form renderer"""

    def render(self, changes):
        """Form renderer"""
        register_views = self.request.registry.getMultiAdapter((self.context, self.request),
                                                               IUserRegistrationViews)
        return {
            'status': 'redirect',
            'location': register_views.password_change_final_view
        }


@pagelet_config(name='password-changed.html',
                layer=IPyAMSLayer)
@template_config(template='templates/password-changed.pt',
                 layer=IPyAMSLayer)
class PasswordChangeFinalView(PortalContextIndexPage):
    """Password change final view"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime, timedelta, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq
from pyramid.decorator import reify
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import implementer

from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS, BOOKING_STATUS_LABEL, IBookingInfo
from pyams_app_msc.feature.booking.zmi import get_booking_element
from pyams_app_msc.feature.booking.zmi.dashboard import BookingStatusQuotationColumn, BookingStatusSeatsColumn
from pyams_app_msc.feature.navigation.interfaces import INavigationViewletManager
from pyams_app_msc.feature.profile.skin import ProfileContextIndexPage, UserMenu
from pyams_app_msc.feature.profile.skin.interfaces import IUserDashboardTable, IUserProfileView
from pyams_app_msc.shared.theater.interfaces import BOOKING_CANCEL_MODE, IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.skin import IPyAMSMSCLayer
from pyams_catalog.query import CatalogResultSet
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_site.interfaces import ISiteRoot
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.date import SH_DATETIME_FORMAT, format_datetime
from pyams_utils.factory import get_interface_base_name
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.table import I18nColumnMixin, Table
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@implementer(IUserDashboardTable)
class BaseUserDashboardTable(Table):
    """Base user dashboard table"""

    object_data = {
        'responsive': True,
        'auto-width': False,
        'paging': False,
        'info': False,
        'row-group': {
            'dataSrc': 0
        },
        'column-defs': [{
            'targets': 0,
            'visible': False
        }],
        'order-fixed': [0, 'asc']
    }


#
# User dashboard
#

@viewlet_config(name='user-dashboard.menu',
                layer=IPyAMSMSCLayer, view=IUserProfileView,
                manager=INavigationViewletManager, weight=30)
class UserDashboardMenu(UserMenu):
    """User dashboard menu"""

    label = _("My bookings")
    href = 'my-dashboard.html'


class UserDashboardTable(BaseUserDashboardTable):
    """User dashboard table"""

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        attributes.setdefault('table', {}).update({
            'data-ams-order': '2,asc'
        })
        return attributes


@pagelet_config(name='my-dashboard.html',
                context=ISiteRoot, layer=IPyAMSLayer)
@implementer(IUserProfileView)
class UserDashboardView(ProfileContextIndexPage):
    """User dashboard view"""

    table = None

    def update(self):
        super().update()
        self.table = UserDashboardTable(self.context, self.request)
        self.table.update()

    def render(self):
        return self.table.render()


@adapter_config(required=(ISiteRoot, IPyAMSMSCLayer, UserDashboardTable),
                provides=IValues)
class UserDashboardTableValues(ContextRequestViewAdapter):
    """User dashboard table values"""

    @property
    def values(self):
        catalog = get_utility(ICatalog)
        params = And(Eq(catalog['object_types'], get_interface_base_name(IBookingInfo)),
                     Eq(catalog['booking_recipient'], self.request.principal.id))
        yield from map(get_booking_element,
                       filter(lambda x: not x.archived,
                              unique_iter(CatalogResultSet(CatalogQuery(catalog).query(params)))))


#
# User archives dashboard
#

@viewlet_config(name='user-archives.menu',
                layer=IPyAMSMSCLayer, view=IUserProfileView,
                manager=INavigationViewletManager, weight=40)
class UserArchivesMenu(UserMenu):
    """User archives dashboard menu"""

    label = _("My archives")
    href = 'my-archives.html'


class UserArchivesTable(BaseUserDashboardTable):
    """User archives table"""

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        attributes.setdefault('table', {}).update({
            'data-ams-order': '2,desc'
        })
        return attributes


@pagelet_config(name='my-archives.html',
                context=ISiteRoot, layer=IPyAMSLayer)
@implementer(IUserProfileView)
class UserArchivesView(ProfileContextIndexPage):
    """User archives view"""

    table = None

    def update(self):
        super().update()
        self.table = UserArchivesTable(self.context, self.request)
        self.table.update()

    def render(self):
        return self.table.render()


@adapter_config(required=(ISiteRoot, IPyAMSMSCLayer, UserArchivesTable),
                provides=IValues)
class UserArchivesTableValues(ContextRequestViewAdapter):
    """User archives table values"""

    @property
    def values(self):
        catalog = get_utility(ICatalog)
        params = And(Eq(catalog['object_types'], get_interface_base_name(IBookingInfo)),
                     Eq(catalog['booking_recipient'], self.request.principal.id))
        yield from map(get_booking_element,
                       filter(lambda x: x.archived,
                              unique_iter(CatalogResultSet(CatalogQuery(catalog).query(params)))))


#
# Dashboard columns
#

@adapter_config(name='theater',
                required=(ISiteRoot, IPyAMSMSCLayer, IUserDashboardTable),
                provides=IColumn)
class UserDashboardTheaterColumn(I18nColumnMixin, GetAttrColumn):
    """User dashboard theater column"""

    i18n_header = _("Theater")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 5

    def get_value(self, obj):
        theater = get_parent(obj.booking, IMovieTheater)
        return get_object_label(theater, self.request)


@adapter_config(name='session-label',
                required=(ISiteRoot, IPyAMSMSCLayer, IUserDashboardTable),
                provides=IColumn)
class UserDashboardSessionLabelColumn(I18nColumnMixin, GetAttrColumn):
    """User dashboard session label column"""

    i18n_header = _("Session")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 10
    responsive_priority = 1

    def get_value(self, obj):
        if obj.entry is None:
            label = obj.session.label
            if not label:
                label = self.request.localizer.translate(_("Out of catalog activity"))
            return label
        return get_object_label(obj.entry, self.request, self.table)


@adapter_config(name='session-date',
                required=(ISiteRoot, IPyAMSMSCLayer, IUserDashboardTable),
                provides=IColumn)
class UserDashboardSessionDateColumn(I18nColumnMixin, GetAttrColumn):
    """User dashboard session date column"""

    i18n_header = _("Date")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 20

    def get_value(self, obj):
        session = obj.session
        return format_datetime(session.start_date, format_string=SH_DATETIME_FORMAT)


@adapter_config(name='status',
                required=(ISiteRoot, IPyAMSMSCLayer, IUserDashboardTable),
                provides=IColumn)
class UserDashboardStatusColumn(I18nColumnMixin, GetAttrColumn):
    """USer dashboard status column"""

    i18n_header = _("Status")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 30

    def get_value(self, obj):
        status = obj.booking.status
        try:
            label = BOOKING_STATUS_LABEL.get(BOOKING_STATUS(status))
        except ValueError:
            label = _("(unknown status)")
        return self.request.localizer.translate(label)


@adapter_config(name='seats',
                required=(ISiteRoot, IPyAMSMSCLayer, IUserDashboardTable),
                provides=IColumn)
class UserDashboardSeatsColumn(BookingStatusSeatsColumn):
    """User dashboard seats column"""

    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 35


@adapter_config(name='quotation',
                required=(ISiteRoot, IPyAMSMSCLayer, IUserDashboardTable),
                provides=IColumn)
class UserDashboardQuotationColumn(BookingStatusQuotationColumn):
    """User dashboard quotation column"""


@adapter_config(name='cancel',
                required=(ISiteRoot, IPyAMSMSCLayer, UserDashboardTable),
                provides=IColumn)
class UserDashboardCancelColumn(GetAttrColumn):
    """User dashboard cancel column"""

    css_classes = {
        'td': 'text-nowrap'
    }
    sortable = 'false'
    weight = 50
    # responsive_priority = 100

    def get_value(self, obj):
        booking = obj.booking
        if booking.status == BOOKING_STATUS.CANCELLED.value:
            return ''
        session = obj.session
        theater = get_parent(session, IMovieTheater)
        settings = IMovieTheaterSettings(theater)
        if settings.booking_cancel_mode == BOOKING_CANCEL_MODE.FORBIDDEN.value:
            return ''
        if settings.booking_cancel_mode == BOOKING_CANCEL_MODE.MAX_DELAY.value:
            dc = IZopeDublinCore(booking)
            last_date = tztime(dc.created) + timedelta(hours=settings.booking_cancel_max_delay)
            if tztime(datetime.now(timezone.utc)) > last_date:
                return ''
        elif settings.booking_cancel_mode == BOOKING_CANCEL_MODE.NOTICE_PERIOD.value:
            last_date = tztime(session.start_date) - timedelta(hours=settings.booking_cancel_notice_period)
            if tztime(datetime.now(timezone.utc)) > last_date:
                return ''
        label = self.request.localizer.translate(_("Cancel..."))
        return f'''<a class="btn btn-sm btn-primary py-0" ''' + \
               f'''   href="{absolute_url(self.context, self.request, 
                                          'cancel-booking.html', 
                                          {'booking_id': ICacheKeyValue(booking)})}">{label}</a>'''



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import Text, TextLine

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class IAddress(Interface):
    """Address interface"""

    street = TextLine(title=_("Number and street"),
                      description=_("Entry number and street name"),
                      required=False)

    locality = TextLine(title=_("Locality"),
                        required=False)

    postal_code = TextLine(title=_("Postal code"),
                           required=True)

    city = TextLine(title=_("City"),
                    required=True)

    comments = Text(title=_("Comments"),
                    description=_("Optional observations which can be added to describe the location"),
                    required=False)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.oauth module

This module is used to handle OAuth authentication, and initialize user profile
on successful login.
"""

import mimetypes

import requests
from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPOk
from zope.lifecycleevent.interfaces import IObjectAddedEvent

from pyams_app_msc.feature.profile import IUserProfile
from pyams_zmi.interfaces.profile import IUserProfile as IZMIProfile

__docformat__ = 'restructuredtext'


try:

    from pyams_auth_oauth.interfaces import IOAuthUser

    @subscriber(IObjectAddedEvent, context_selector=IOAuthUser)
    def handle_new_oauth_user(event):
        """Handle new OAuth user"""
        folder = event.newParent
        if folder is None:
            return
        user_info = event.object
        principal_info = folder.get_principal(principal_id=f'{folder.prefix}:{user_info.user_id}', info=True)
        if principal_info is None:
            return
        profile = IUserProfile(principal_info)
        profile.firstname = user_info.first_name
        profile.lastname = user_info.last_name
        profile.email = user_info.email
        if user_info.picture:
            response = requests.get(user_info.picture)
            if response.status_code == HTTPOk.code:
                zmi_profile = IZMIProfile(principal_info)
                content_type = response.headers.get('content-type')
                if content_type:
                    filename = f'profile{mimetypes.guess_extension(content_type)}'
                else:
                    filename = 'profile.jpg'
                zmi_profile.avatar = (filename, response.content)

except ImportError:
    pass



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.shared.theater.portlet import IMovieTheaterCarouselPortletSettings
from pyams_app_msc.shared.theater.portlet.skin.interfaces import IMovieTheaterCarouselPortletRendererSettings
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@factory_config(provided=IMovieTheaterCarouselPortletRendererSettings)
class MovieTheaterCarouselPortletRendererSettings(Persistent, Contained):
    """Movie theater carousel portlet renderer settings"""

    css_class = FieldProperty(IMovieTheaterCarouselPortletRendererSettings['css_class'])
    cards_css_class = FieldProperty(IMovieTheaterCarouselPortletRendererSettings['cards_css_class'])
    thumb_selection = FieldProperty(IMovieTheaterCarouselPortletRendererSettings['thumb_selection'])
    automatic_slide = FieldProperty(IMovieTheaterCarouselPortletRendererSettings['automatic_slide'])
    fade_effect = FieldProperty(IMovieTheaterCarouselPortletRendererSettings['fade_effect'])
    display_controls = FieldProperty(IMovieTheaterCarouselPortletRendererSettings['display_controls'])
    enable_touch = FieldProperty(IMovieTheaterCarouselPortletRendererSettings['enable_touch'])


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface, IMovieTheaterCarouselPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/carousel-default.pt', layer=IPyAMSLayer)
class MovieTheaterCarouselPortletRenderer(PortletRenderer):
    """Movie theater carousel portlet renderer"""

    label = _("MSC: Movie theaters carousel (default)")
    weight = 1

    settings_interface = IMovieTheaterCarouselPortletRendererSettings

    @staticmethod
    def get_location(theater):
        address = theater.address
        return address.city if address is not None else None



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.tmdb.api.schema module

This module defines the schema definition for TMDB API service.
"""

from colander import MappingSchema, SchemaNode, String

from pyams_utils.rest import BaseResponseSchema

__docformat__ = 'restructuredtext'


class MovieSearchQuery(MappingSchema):
    """Movie search schema"""
    term = SchemaNode(String(),
                      description="Query string")


class MovieSearchRequest(MappingSchema):
    """Movie search request schema"""
    querystring = MovieSearchQuery()


class MovieItem(MappingSchema):
    """Movie item schema"""
    id = SchemaNode(String(),
                    description="Movie ID")
    text = SchemaNode(String(),
                      description="Movie title and year")


class MovieList(BaseResponseSchema):
    """Movies list schema"""
    result = MovieItem()


class MovieSearchResponse(MappingSchema):
    """Movie search response"""
    body = MovieList()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

import logging
from importlib import import_module

from hypatia.interfaces import ICatalog

from pyams_app_msc.feature.booking.interfaces import IBookingInfo
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.interfaces import MSC_CONTRIBUTOR_ROLE, MSC_MANAGER_ROLE, MSC_OPERATOR_ROLE, MSC_READER_ROLE
from pyams_app_msc.reference.structure.interfaces import IStructureTypeTable
from pyams_app_msc.shared.catalog.interfaces import IWfCatalogEntry
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.audience import ICinemaAudienceContainer
from pyams_catalog.generations import check_required_indexes
from pyams_catalog.index import DatetimeIndexWithInterface, FieldIndexWithInterface, KeywordIndexWithInterface
from pyams_catalog.interfaces import MINUTE_RESOLUTION
from pyams_content.generations import check_required_tables, check_required_tools
from pyams_security.index import PrincipalsRoleIndex
from pyams_site.generations import check_required_utilities
from pyams_site.interfaces import ISiteGenerations
from pyams_utils.registry import utility_config

LOGGER = logging.getLogger('PyAMS (MSC)')


REQUIRED_UTILITIES = ()

REQUIRED_TABLES = (
    (IStructureTypeTable, 'structures-types'),
)

REQUIRED_TOOLS = (
)

REQUIRED_INDEXES = (
    ('role:msc:manager', PrincipalsRoleIndex, {
        'role_id': MSC_MANAGER_ROLE
    }),
    ('role:msc:operator', PrincipalsRoleIndex, {
        'role_id': MSC_OPERATOR_ROLE
    }),
    ('role:msc:contributor', PrincipalsRoleIndex, {
        'role_id': MSC_CONTRIBUTOR_ROLE
    }),
    ('role:msc:reader', PrincipalsRoleIndex, {
        'role_id': MSC_READER_ROLE
    }),
    ('catalog_audience', KeywordIndexWithInterface, {
        'interface': IWfCatalogEntry,
        'discriminator': 'audiences'
    }),
    ('planning_start_date', DatetimeIndexWithInterface, {
        'interface': ISession,
        'discriminator': 'start_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('planning_end_date', DatetimeIndexWithInterface, {
        'interface': ISession,
        'discriminator': 'end_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('planning_room', FieldIndexWithInterface, {
        'interface': ISession,
        'discriminator': 'room'
    }),
    ('planning_audience', KeywordIndexWithInterface, {
        'interface': ISession,
        'discriminator': 'audiences'
    }),
    ('booking_session', FieldIndexWithInterface, {
        'interface': IBookingInfo,
        'discriminator': 'session_index'
    }),
    ('booking_recipient', FieldIndexWithInterface, {
        'interface': IBookingInfo,
        'discriminator': 'recipient'
    }),
    ('booking_status', FieldIndexWithInterface, {
        'interface': IBookingInfo,
        'discriminator': 'status'
    })
)


def check_required_facets(site, catalog_name=''):
    """Check facets index for required audiences values"""
    sm = site.getSiteManager()
    catalog = sm.queryUtility(ICatalog, name=catalog_name)
    if catalog is None:
        LOGGER.warning("No catalog found! Facets index check ignored...")
        return
    index = catalog.get('facets')
    if index is None:
        LOGGER.warning("No facets index found! Facets index check ignored...")
    facets = index.facets
    for name, theater in sm.getUtilitiesFor(IMovieTheater):
        audiences = ICinemaAudienceContainer(theater, None)
        for audience in audiences:
            facet_name = f'audience:{audience}'
            if facet_name not in facets:
                facets.add(facet_name)
    index.facets = facets


@utility_config(name='PyAMS MSC application', provides=ISiteGenerations)
class MSCGenerationsChecker:
    """MSC package generations checker"""

    order = 200
    generation = 2

    def evolve(self, site, current=None):
        """Check for required utilities, tables and tools"""
        check_required_utilities(site, REQUIRED_UTILITIES)
        check_required_tables(site, REQUIRED_TABLES)
        check_required_tools(site, REQUIRED_TOOLS)
        check_required_indexes(site, REQUIRED_INDEXES)
        check_required_facets(site, catalog_name='')

        if not current:
            current = 1
        for generation in range(current, self.generation):
            module_name = f'pyams_app_msc.generations.evolve{generation}'
            module = import_module(module_name)
            module.evolve(site)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.zmi.interfaces import IMovieTheaterCalendarPresentationMenu, \
    IMovieTheaterCatalogPresentationMenu, IMovieTheaterMoviesPresentationMenu
from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.interfaces import MANAGE_TEMPLATE_PERMISSION
from pyams_portal.zmi.interfaces import IPortalContextPresentationMenu
from pyams_portal.zmi.presentation import PortalContextPresentationEditForm, PortalContextTemplateLayoutMenu, \
    PortalContextTemplateLayoutView
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

from pyams_app_msc import _


#
# Movie theater calendar presentation components
#

@viewletmanager_config(name='calendar-presentation.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=IPortalContextPresentationMenu, weight=30,
                       provides=IMovieTheaterCalendarPresentationMenu,
                       permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCalendarPresentationMenu(NavigationMenuItem):
    """Movie theater calendar presentation menu"""

    label = _("Calendar presentation")
    icon_class = 'fas fa-calendar'
    href = '#calendar-presentation.html'


@ajax_form_config(name='calendar-presentation.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCalendarPresentationEditForm(PortalContextPresentationEditForm):
    """Movie theater calendar presentation edit form"""

    title = _("Calendar template configuration")

    page_name = 'calendar'


@viewlet_config(name='calendar-template-layout.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IMovieTheaterCalendarPresentationMenu, weight=10,
                permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCalendarTemplateLayoutMenu(PortalContextTemplateLayoutMenu):
    """Movie theater calendar template layout menu"""

    label = _("Calendar layout")
    href = '#calendar-template-layout.html'

    page_name = 'calendar'


@pagelet_config(name='calendar-template-layout.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCalendarTemplateLayoutView(PortalContextTemplateLayoutView):
    """Movie theater calendar template layout view"""

    page_name = 'calendar'


#
# Movie theater movies list presentation components
#

@viewletmanager_config(name='movies-presentation.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=IPortalContextPresentationMenu, weight=35,
                       provides=IMovieTheaterMoviesPresentationMenu,
                       permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterMoviesPresentationMenu(NavigationMenuItem):
    """Movie theater movies presentation menu"""

    label = _("Movies list presentation")
    icon_class = 'fas fa-photo-film'
    href = '#movies-presentation.html'


@ajax_form_config(name='movies-presentation.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterMoviesPresentationEditForm(PortalContextPresentationEditForm):
    """Movie theater movies presentation edit form"""

    title = _("Movies list template configuration")

    page_name = 'movies'


@viewlet_config(name='movies-template-layout.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IMovieTheaterMoviesPresentationMenu, weight=10,
                permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterMoviesTemplateLayoutMenu(PortalContextTemplateLayoutMenu):
    """Movie theater movies list template layout menu"""

    label = _("Movies list layout")
    href = '#movies-template-layout.html'

    page_name = 'movies'


@pagelet_config(name='movies-template-layout.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterMoviesTemplateLayoutView(PortalContextTemplateLayoutView):
    """Movie theater movies list template layout view"""

    page_name = 'movies'


#
# Catalog entries presentation components
#

@viewletmanager_config(name='catalog-presentation.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=IPortalContextPresentationMenu, weight=40,
                       provides=IMovieTheaterCatalogPresentationMenu,
                       permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCatalogPresentationMenu(NavigationMenuItem):
    """Movie theater catalog presentation menu"""

    label = _("Catalog presentation")
    icon_class = 'fas fa-panorama'
    href = '#catalog-presentation.html'


@ajax_form_config(name='catalog-presentation.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCatalogPresentationEditForm(PortalContextPresentationEditForm):
    """Portal context footer presentation edit form"""

    title = _("Catalog template configuration")

    page_name = 'catalog'


@viewlet_config(name='catalog-template-layout.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IMovieTheaterCatalogPresentationMenu, weight=10,
                permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCatalogTemplateLayoutMenu(PortalContextTemplateLayoutMenu):
    """Movie theater catalog template layout menu"""

    label = _("Catalog layout")
    href = '#catalog-template-layout.html'

    page_name = 'catalog'


@pagelet_config(name='catalog-template-layout.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
class MovieTheaterCatalogTemplateLayoutView(PortalContextTemplateLayoutView):
    """Movie theater catalog template layout view"""

    page_name = 'catalog'



#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""
This module contains PyAMS MSC application package
"""
import os
from setuptools import setup, find_packages


DOCS = os.path.join(os.path.dirname(__file__),
                    'docs')

README = os.path.join(DOCS, 'README.rst')
HISTORY = os.path.join(DOCS, 'HISTORY.rst')

version = '1.99.10'
long_description = open(README).read() + '\n\n' + open(HISTORY).read()

data_dir = 'pkg'
data_files = [(d, [os.path.join(d, f) for f in files])
              for d, folders, files in os.walk(data_dir)]

tests_require = [
    'pyams_content_themes',
    'pyramid_zcml',
    'zope.exceptions'
]

setup(name='pyams_app_msc',
      version=version,
      description="PyAMS application for cinema reservation management",
      long_description=long_description,
      classifiers=[
          "License :: OSI Approved :: Zope Public License",
          "Development Status :: 4 - Beta",
          "Programming Language :: Python",
          "Framework :: Pyramid",
          "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      keywords='Pyramid PyAMS',
      author='Thierry Florac',
      author_email='tflorac@ulthar.net',
      url='https://pyams.readthedocs.io',
      license='ZPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=[],
      include_package_data=True,
      package_data={
          '': ['*.rst', '*.txt', '*.pt',
               '*.pot', '*.po', '*.mo',
               '*.png', '*.gif', '*.jpeg', '*.jpg', '*.svg',
               '*.ttf', '*.woff2',
               '*.scss', '*.css', '*.js', '*.map']
      },
      data_files=data_files,
      zip_safe=False,
      python_requires='>=3.7',
      # uncomment this to be able to run tests with setup.py
      # test_suite="pyams_app_msc.tests.test_utilsdocs.test_suite",
      tests_require=tests_require,
      extras_require=dict(test=tests_require),
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'colander',
          'cornice',
          'elasticsearch_dsl',
          'fanstatic',
          'hypatia',
          'persistent',
          'pillow',
          'pyams_catalog >= 2.1.0',
          'pyams_content >= 1.99.9',
          'pyams_content_api',
          'pyams_content_es',
          'pyams_file',
          'pyams_form >= 2.1.0',
          'pyams_i18n',
          'pyams_layer',
          'pyams_mail',
          'pyams_pagelet',
          'pyams_portal',
          'pyams_scheduler',
          'pyams_security >= 2.2.1',
          'pyams_security_views',
          'pyams_sequence',
          'pyams_site',
          'pyams_skin >= 2.2.3',
          'pyams_table',
          'pyams_utils >= 2.3.1',
          'pyams_viewlet',
          'pyams_workflow',
          'pyams_zmi >= 2.3.1',
          'pyams_zmq',
          'pyramid',
          'pyramid_mailer',
          'reportlab',
          'requests',
          'transaction',
          'ZODB',
          'zope.annotation',
          'zope.container',
          'zope.copy',
          'zope.dublincore',
          'zope.interface',
          'zope.intid',
          'zope.lifecycleevent',
          'zope.location',
          'zope.principalannotation',
          'zope.schema',
          'zope.traversing'
      ],
      entry_points={
          'fanstatic.libraries': [
              'msc = pyams_app_msc.zmi:library',
              'mscapp = pyams_app_msc.skin:library'
          ]
      })



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_app_msc.shared.catalog.interfaces import ICatalogManager
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.shared.common.zmi.interfaces import IWorkflowDeleteFormTarget
from pyams_content.shared.common.zmi.workflow import SharedContentDeleteForm
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_zmi.interfaces import IAdminLayer


@adapter_config(required=(ICatalogManager, IAdminLayer, SharedContentDeleteForm),
                provides=IWorkflowDeleteFormTarget)
def catalog_entry_workflow_delete_form_target(context, request, form):
    """Catalog entry workflow delete form target"""
    return get_parent(context, IMovieTheater)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_app_msc.reference.structure.interfaces import IStructureType, IStructureTypeTable
from pyams_app_msc.reference.structure.zmi.table import StructureTypeTableContainerTable
from pyams_content.interfaces import MANAGE_REFERENCE_TABLE_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.view import IModalEditForm, IModalPage
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.intids import IUniqueID
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.event import get_json_table_row_add_callback, get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='add-structure-type.menu',
                context=IStructureTypeTable, layer=IAdminLayer, view=StructureTypeTableContainerTable,
                manager=IToolbarViewletManager, weight=10,
                permission=MANAGE_REFERENCE_TABLE_PERMISSION)
class StructureTypeAddAction(ContextAddAction):
    """StructureType add action"""

    label = _("Add structure type")
    href = 'add-structure-type.html'


@ajax_form_config(name='add-structure-type.html',
                  context=IStructureTypeTable, layer=IPyAMSLayer,
                  permission=MANAGE_REFERENCE_TABLE_PERMISSION)
class StructureTypeAddForm(AdminModalAddForm):
    """Structure type add form"""

    subtitle = _("New structure type")
    legend = _("New structure type properties")
    modal_class = 'modal-xl'

    fields = Fields(IStructureType).select('title')
    content_factory = IStructureType

    def add(self, obj):
        oid = IUniqueID(obj).oid
        self.context[oid] = obj


@adapter_config(required=(IStructureTypeTable, IAdminLayer, IModalPage),
                provides=IFormTitle)
def structure_type_add_form_title(context, request, view):
    """Structure type add form title"""
    return get_object_label(context, request, view)


@adapter_config(required=(IStructureTypeTable, IAdminLayer, StructureTypeAddForm),
                provides=IAJAXFormRenderer)
class StructureTypeAddFormAJAXRenderer(ContextRequestViewAdapter):
    """Structure type add form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        table = get_parent(self.context, IStructureTypeTable)
        return {
            'callbacks': [
                get_json_table_row_add_callback(table, self.request,
                                                StructureTypeTableContainerTable, changes)
            ]
        }


@adapter_config(required=(IStructureType, IAdminLayer, Interface),
                provides=ITableElementEditor)
class StructureTypeElementEditor(TableElementEditor):
    """Structure type table element editor"""


@ajax_form_config(name='properties.html',
                  context=IStructureType, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class StructureTypeEditForm(AdminModalEditForm):
    """Structure type properties edit form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("Structure type: {}")).format(
            II18n(self.context).query_attribute('title', request=self.request))

    legend = _("Structure type properties")
    modal_class = 'modal-xl'

    fields = Fields(IStructureType).select('title')


@adapter_config(required=(IStructureType, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def structure_type_edit_form_title(context, request, form):
    """Structure type edit form title"""
    table = get_utility(IStructureTypeTable)
    return TITLE_SPAN.format(get_object_label(table, request, form))


@adapter_config(required=(IStructureType, IAdminLayer, StructureTypeEditForm),
                provides=IAJAXFormRenderer)
class StructureTypeEditFormAJAXRenderer(ContextRequestViewAdapter):
    """Structure type edit form AJAX renderer"""

    def render(self, changes):
        """AJAX result renderer"""
        if not changes:
            return None
        table = get_parent(self.context, IStructureTypeTable)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(table, self.request,
                                                    StructureTypeTableContainerTable, self.context)
            ]
        }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_app_msc.shared.catalog.interfaces import ICatalogManager, ICatalogManagerTarget
from pyams_content.shared.common.portal import shared_content_portal_page
from pyams_portal.interfaces import IPortalPage, IPortalPortletsConfiguration
from pyams_portal.page import portal_context_portlets_configuration
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent


@adapter_config(required=IWfCatalogEntry,
                provides=IPortalPage)
@adapter_config(name='catalog',
                required=IWfCatalogEntry,
                provides=IPortalPage)
def catalog_entry_page_factory(context):
    """Catalog entry page factory"""
    return shared_content_portal_page(context, page_name='catalog')


@adapter_config(name='catalog',
                required=IWfCatalogEntry,
                provides=IPortalPortletsConfiguration)
def catalog_entry_portlets_configuration(context):
    """Movie theater portlets configuration"""
    return portal_context_portlets_configuration(context, page_name='catalog')


@adapter_config(name='catalog',
                required=ICatalogManager,
                provides=IPortalPortletsConfiguration)
def catalog_manager_portlets_configuration(context):
    """Catalog manager portlets configuration"""
    parent = get_parent(context, ICatalogManagerTarget)
    return portal_context_portlets_configuration(parent, page_name='catalog')



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.audience import ICinemaAudienceContainer
from pyams_app_msc.shared.theater.portlet.interfaces import IAudiencesPortletSettings
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _
from pyams_utils.traversing import get_parent

AUDIENCES_PORTLET_NAME = 'msc.portlet.audiences'
AUDIENCES_ICON_CLASS = 'fas fa-bullhorn'


@factory_config(IAudiencesPortletSettings)
class AudiencesPortletSettings(PortletSettings):
    """Audiences banner portlet settings"""

    @staticmethod
    def get_parent(context):
        """Parent movie theater getter"""
        return get_parent(context, IMovieTheater)

    @staticmethod
    def get_audiences(context):
        """Audiences list getter"""
        theater = get_parent(context, IMovieTheater)
        audiences = ICinemaAudienceContainer(theater, None)
        if audiences is not None:
            yield from audiences.get_active_items()


@portlet_config(permission=None)
class AudiencesPortlet(Portlet):
    """Audiences portlet"""

    name = AUDIENCES_PORTLET_NAME
    label = _("MSC: Audiences list")

    settings_factory = IAudiencesPortletSettings
    toolbar_css_class = AUDIENCES_ICON_CLASS



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from collections import OrderedDict
from enum import Enum

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.schema import Bool, Choice, Datetime, Int, List, Text, TextLine, Timedelta
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_app_msc.shared.theater.interfaces.audience import AUDIENCES_VOCABULARY
from pyams_app_msc.shared.theater.interfaces.price import PRICES_VOCABULARY
from pyams_app_msc.shared.theater.interfaces.room import ROOMS_VOCABULARY
from pyams_sequence.schema import InternalReferenceField

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class VERSION_INFO(Enum):
    """Version information"""
    OV = 'ov'
    OVWS = 'ovws'
    TF = 'tf'


VERSION_INFO_LABEL = OrderedDict((
    (VERSION_INFO.OV, _("Original version")),
    (VERSION_INFO.OVWS, _("Original version with subtitles")),
    (VERSION_INFO.TF, _("Translated version"))
))

VERSION_INFO_ABBR = {
    VERSION_INFO.OV: _("OV"),
    VERSION_INFO.OVWS: _("OVWS"),
    VERSION_INFO.TF: _("TV")
}


VERSION_INFO_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in VERSION_INFO_LABEL.items()
])


class ISession(IAttributeAnnotatable):
    """Session interface"""

    def get_target(self):
        """Planning target getter"""

    label = TextLine(title=_("Label"),
                     description=_("If no label is set, activity label will be used"),
                     required=False)

    def get_label(self):
        """Session label getter"""

    start_date = Datetime(title=_("Session start date"),
                          description=_("Start date and time of the session"),
                          required=True)

    duration = Timedelta(title=_("Duration"),
                         description=_("Session duration, given as time delta object"),
                         required=True)

    end_date = Datetime(title=_("Session end date"),
                        description=_("End date and time of the session"),
                        required=True)

    room = Choice(title=_("Session room"),
                  description=_("Theater room in which session is planned"),
                  vocabulary=ROOMS_VOCABULARY,
                  required=True)

    def get_room(self):
        """Room getter"""

    capacity = Int(title=_("Session capacity"),
                   description=_("Number of places available for this session"),
                   required=False)

    version = Choice(title=_("Displayed version"),
                     vocabulary=VERSION_INFO_VOCABULARY,
                     required=False)

    audiences = List(title=_("Selected audiences"),
                     description=_("List of audiences selected for this session"),
                     value_type=Choice(vocabulary=AUDIENCES_VOCABULARY),
                     required=False)

    temporary = Bool(title=_("Temporary session"),
                     description=_("If 'yes', this session is 'temporary' and not confirmed yet!"),
                     required=True,
                     default=False)

    bookable = Bool(title=_("Open for internal booking"),
                    description=_("Only internal bookable sessions are visible to operators for booking"),
                    required=True,
                    default=True)
    
    extern_bookable = Bool(title=_("Open for external booking"),
                           description=_("Only external bookable sessions are visible to public for booking"),
                           required=True,
                           default=False)

    comments = Text(title=_("Movie theater comments"),
                    description=_("These comments will be displayed to the users on booking form"),
                    required=False)

    notepad = Text(title=_("Notepad"),
                   description=_("These comments are for internal use only"),
                   required=False)


class IMovieTheaterSession(ISession):
    """Movie theater session interface"""

    activity = InternalReferenceField(title=_("Activity"),
                                      description=_("You can select an existing activity from your catalog, or keep "
                                                    "this input empty if you just want to mark a period as not being "
                                                    "available for booking"),
                                      required=False)


PLANNING_ANNOTATION_KEY = 'msc.planning'


class IPlanning(IContainer):
    """Planning container interface"""

    contains(ISession)

    def add_session(self, session):
        """Add new session to container"""


class IPlanningTarget(IAttributeAnnotatable):
    """Planning target marker interface"""


class IWfPlanningTarget(IPlanningTarget):
    """Workflow managed planning target"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface, alsoProvides, implementer

from pyams_app_msc.interfaces import VIEW_CATALOG_PERMISSION
from pyams_app_msc.shared.catalog.interfaces import ICatalogManagerTarget
from pyams_app_msc.shared.catalog.zmi.interfaces import ICatalogManagementView
from pyams_content.shared.common.zmi.dashboard import SharedToolAllInterventionsMenu, \
    SharedToolArchivedContentsView, SharedToolDashboardMenu, SharedToolDashboardView, SharedToolLastModificationsView, \
    SharedToolLastPublicationsView, SharedToolPreparationsView, SharedToolPublicationsView, \
    SharedToolRetiredContentsView, SharedToolSubmissionsView
from pyams_content.zmi.interfaces import IAllDashboardMenu
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.adapter import adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IMenuHeader, INavigationViewletManager
from pyams_zmi.zmi.viewlet.menu import ContentManagementMenu

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


#
# Activities catalog dashboard
#

@viewletmanager_config(name='content-manager.menu',
                       context=ICatalogManagerTarget, layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=100,
                       provides=IContentManagementMenu)
class CatalogManagementMenu(ContentManagementMenu):
    """Catalog management menu"""

    _header = _("Catalog management")


class CatalogManagementDashboardViewMixin:
    """Catalog management dashboard mixin view"""

    def get_table(self, container):
        table = super().get_table(container)
        alsoProvides(table, ICatalogManagementView)
        return table


@adapter_config(required=(ICatalogManagerTarget, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def catalog_manager_menu_header(context, request, view, menu):
    """Catalog manager menu header"""
    return request.localizer.translate(_("Activities catalog"))


@viewlet_config(name='dashboard.menu',
                context=ICatalogManagerTarget, layer=IAdminLayer,
                manager=IContentManagementMenu, weight=5,
                permission=VIEW_CATALOG_PERMISSION)
class CatalogManagerDashboardMenu(SharedToolDashboardMenu):
    """Catalog manager dashboard menu"""


@pagelet_config(name='dashboard.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_CATALOG_PERMISSION)
@implementer(ICatalogManagementView)
class CatalogManagerDashboardView(SharedToolDashboardView):
    """Catalog manager dashboard view"""


@pagelet_config(name='my-preparations.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_CATALOG_PERMISSION)
class CatalogManagerPreparationsView(CatalogManagementDashboardViewMixin, SharedToolPreparationsView):
    """Catalog manager preparations view"""


@pagelet_config(name='my-submissions.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class CatalogManagerSubmissionsView(CatalogManagementDashboardViewMixin, SharedToolSubmissionsView):
    """Catalog manager submissions view"""


@pagelet_config(name='my-publications.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class CatalogManagerPublicationsView(CatalogManagementDashboardViewMixin, SharedToolPublicationsView):
    """Catalog manager publications view"""


@pagelet_config(name='my-retired-contents.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class CatalogManagerRetiredContentsView(CatalogManagementDashboardViewMixin, SharedToolRetiredContentsView):
    """Catalog manager retired contents view"""


@pagelet_config(name='my-archived-contents.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class CatalogManagerArchivedContentsView(CatalogManagementDashboardViewMixin, SharedToolArchivedContentsView):
    """Catalog manager archived contents view"""


@viewletmanager_config(name='all-interventions.menu',
                       context=ICatalogManagerTarget, layer=IAdminLayer,
                       manager=IContentManagementMenu, weight=20,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IAllDashboardMenu)
class CatalogManagerAllInterventionsMenu(SharedToolAllInterventionsMenu):
    """Catalog manager 'all interventions' dashboard menu"""

    css_class = ''


@pagelet_config(name='last-published.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class CatalogManagerLastPublicationsView(CatalogManagementDashboardViewMixin, SharedToolLastPublicationsView):
    """Catalog manager last publications view"""


@pagelet_config(name='last-modified.html',
                context=ICatalogManagerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class CatalogManagerLastModificationsView(CatalogManagementDashboardViewMixin, SharedToolLastModificationsView):
    """Catalog manager last modifications view"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from urllib.parse import urlencode

from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.skin.interfaces import IPublicURL
from pyams_layer.interfaces import IPyAMSLayer
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.interfaces.url import ICanonicalURL
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url


@adapter_config(required=(IWfCatalogEntry, IPyAMSLayer),
                provides=ICanonicalURL)
class CatalogEntryCanonicalURL(ContextRequestAdapter):
    """Catalog entry canonical URL"""

    def get_url(self, view_name=None, query=None):
        theater = get_parent(self.context, IMovieTheater)
        query = urlencode(query) if query else None
        return absolute_url(theater, self.request,
                            f"+/{ISequentialIdInfo(self.context).get_base_oid().strip()}"
                            f"::{self.context.content_url}"
                            f"{'/{}'.format(view_name) if view_name else '.html'}"
                            f"{'?{}'.format(query) if query else ''}")


@adapter_config(name='booking-new.html',
                required=(IWfCatalogEntry, IPyAMSLayer),
                provides=IPublicURL)
class CatalogEntryPublicURL(ContextRequestAdapter):
    """Catalog entry public URL"""

    def get_url(self):
        """Public URL getter"""
        return self.request.url
